#!/usr/bin/python

try:
    from breve import Template
    from breve.tags import html
    from breve.flatten import register_flattener
except ImportError:
    print 'run "python setup.py install" from the parent directory first'
    raise SystemExit


Template.tidy = True

########## test basic functionality
from datetime import datetime

def flatten_datetime ( o ):
    return o.strftime ( '%Y/%m/%d' )
register_flattener ( datetime, flatten_datetime )

def example_renderer ( tag, data ):
    return tag ( class_ = 'custom-renderer' ) [
        'This was generated by a custom renderer.',
        'We also passed in this data: ', data
    ]

vars = dict ( 
    today = datetime.today ( ),
    message = 'Hello, world!',
    example_renderer = example_renderer,
    username = 'admin'
)

for root, template in [ ( '1_basics', 'index' ),
                        ( '2_includes', 'index' ),
                        ( '3_inheritance', 'fragment' ),
                        ( '4_flatteners', 'index' ),
                        ( '5_conditionals', 'index' ),
                        ( '6_dynamic_inheritance', 'fragment' ),
                        ( '7_escape_artist', 'index' ) ]:
    print "RUNNING EXAMPLE", root, template
    print "=" * 40
    t = Template ( html.tags, root = root, doctype = html.doctype, xmlns = html.xmlns )
    print t.render ( template = template, vars = vars )
    print "\n\n\n"


####### test custom tags
import sys
sys.path.insert ( 0, '.' )
import sitemap # our custom tag definitions

vars = dict ( loc = 'http://www.example.com/',
              lastmod = '2007-01-01',
              changefreq = 'monthly',
              priority = 0.8 )
root, template = ( '8_custom_tags', 'index' )
print "RUNNING EXAMPLE", root, template
print "=" * 40

t = Template ( tags = sitemap.tags, xmlns = sitemap.xmlns, doctype = sitemap.doctype, root = root )
t.namespace = 'v'
print t.render ( template = template, vars = vars )
print "\n\n\n"


####### test it all!
root = 'A_all_features'

vars = dict (
    message = 'Hello, World.'
)

for template in [ 'include/index' ]:
    print "RUNNING EXAMPLE", root, template
    print "=" * 40

    t = Template ( tags = html.tags, xmlns = html.xmlns, doctype = html.doctype, root = root )
    t.namespace = 'v'
    print t.render ( template = template, vars = vars )
    print "\n\n\n"

